export interface SimpleCache<T> {
    get: (key: string) => T | undefined | Promise<Awaited<T> | undefined>;
    set: (key: string, value: T, options?: {
        secondsUntilExpiration?: number;
    }) => void | Promise<void>;
}
export declare type KeySerializationMethod<P> = (args: P) => string;
export declare const noOp: <LR, CR>(value: LR) => CR;
export declare const defaultKeySerializationMethod: {
    (value: any, replacer?: ((this: any, key: string, value: any) => any) | undefined, space?: string | number | undefined): string;
    (value: any, replacer?: (string | number)[] | null | undefined, space?: string | number | undefined): string;
};
export declare const defaultValueSerializationMethod: <LR, CR>(value: LR) => CR;
export declare type SimpleCacheResolutionMethod<L extends (...args: any[]) => any, CR extends any> = (args: {
    fromInput: Parameters<L>;
}) => SimpleCache<CR>;
export declare const getCacheFromCacheOption: <L extends (...args: any[]) => any, CR extends unknown>({ forInput, cacheOption, }: {
    forInput: Parameters<L>;
    cacheOption: SimpleCache<CR> | SimpleCacheResolutionMethod<L, CR>;
}) => SimpleCache<CR>;
export interface WithSimpleCachingOptions<
/**
 * the response returned by the wrapped logic for the input
 */
LR extends any, 
/**
 * the response returned by the cache for the key
 */
CR extends any, 
/**
 * the logic we are caching the responses for
 */
L extends (...args: any[]) => LR> {
    cache: SimpleCache<CR> | SimpleCacheResolutionMethod<L, CR>;
    serialize?: {
        key?: KeySerializationMethod<Parameters<L>>;
        value?: (returned: LR) => CR;
    };
    deserialize?: {
        value?: (cached: CR) => LR;
    };
    secondsUntilExpiration?: number;
    hook?: {
        onSet: (args: {
            forInput: Parameters<L>;
            forKey: string;
            value: {
                output: ReturnType<L>;
                cached: CR;
            };
        }) => void;
    };
}
/**
 * caches the promise of each invocation, based on serialization of the inputs.
 *
 * by default, this uses the `simple-in-memory-cache`, but you can pass in a `get` and `set` method for any persistent storage (e.g., local storage, cookie storage, dynamodb, etc)
 *
 * for example:
 * ```ts
 * const getApiResult = withSimpleCaching(({ name, number }) => axios.get(URL, { name, number }));
 * const result1 = getApiResult({ name: 'casey', number: 821 }); // calls the api, puts promise of results into cache, returns that promise
 * const result2 = getApiResult({ name: 'casey', number: 821 }); // returns the same promise from above, because it was found in cache - since same input as request above was used
 * expect(result1).toBe(result2); // same exact object - the promise
 * expect(await result1).toBe(await result2); // same exact object - the result of the promise
 * ```
 */
export declare const withSimpleCaching: <LR extends unknown, CR extends unknown, L extends (...args: any[]) => LR>(logic: L, { cache: cacheOption, serialize: { key: serializeKey, value: serializeValue, }, deserialize: { value: deserializeValue }, secondsUntilExpiration, hook, }: WithSimpleCachingOptions<LR, CR, L>) => L;
